from abc import ABC
from string import Template
from os.path import isfile
from os.path import splitext
from datetime import datetime

from core import env

class FileTemplate(ABC):

    file_open = None

    def __init__(self, file_name:str):
        """
        Template será aplicado no arquivo escolhido.
        :param file_name: nome do arquivo escolhido.
        """
        self.file_type = splitext(file_name)[1][1:]
        if not self.file_type:
            raise Exception('Extensão de arquivo não informada')

        self.file_path = f'/srv/asc/template/{self.file_type}/' \
                           f'{env.get("api_server_signature")}-' \
                           f'{file_name}'

        if not isfile(self.file_path):
            raise FileNotFoundError(self.file_path)

        self.file_data = ''
        self.file_open = open(self.file_path, 'r')
        self.file_read = self.file_open.read()
        self.file_vars = Template(self.file_read)
        self.update(
            #ASC_VERSION='.'.join(VERSION),
            SYS_DATE=datetime.now().strftime('%d/%m/%Y %H:%M:%S'),
            EXT_FILE=self.file_type
        )

    def update(self, merge_vars=True, **kwargs):
        """
        Atualiza variáveis do arquivo.
        :param merge_vars: gerar um novo template a partir do novo valor.
        :param kwargs:
        :return:
        """
        self.file_data = self.file_vars.safe_substitute(kwargs)

        if merge_vars:
            self.file_vars = Template(self.file_data)

        return self.file_data

    def export(self, absolute_path:str):
        file = open(absolute_path, 'w')
        file.write(self.file_data)
        file.close()

    def __del__(self):
        self.file_open.close()
