from asc.core import Base, env
from sqlalchemy import Sequence
from sqlalchemy.schema import CreateTable

metadata = Base.metadata


class Model(Base):
    """ Classe base responsável por representar um
    modelo de dados abstrato. """

    __abstract__ = True
    __table_args__ = {
        'schema': env.get('database__models_schema','private')
    }

    def as_dict(self):
        return {c.name: getattr(self, c.name) for c in self.__table__.columns}

    def get_sequence(self,name:str) -> Sequence:
        return Sequence(f'{self.__tablename__}_{name}_seq')

    @classmethod
    def schema(cls, schema):
        if isinstance(cls.__table_args__, dict):
            cls.__table_args__.update({'schema': schema})
        else:
            cls.__table_args__ += ({'schema': schema},)
        return cls

    @classmethod
    def create_table(self, engine):
        """ Cria modelo da base. """
        print(CreateTable(self.__table__,if_not_exists=True).compile(bind=engine))

    def update_db(self, **fields):
        """ método responsável por realizar atribuição da
        classe com os campos obtidos em um dicionario. """
        for key, value in fields.items():
            if hasattr(self, key):
                setattr(self, key, value)

    def __repr__(self):
        return f"<{type(self).__name__}>"

# end-of-file