from dataclasses import dataclass

from asc.core.ABC.Serializer import DataSerializer

@dataclass
class Address:
    pass


@dataclass
class Phone:
    number:str = '900000000'
    area_code:int = 0


@dataclass
class Customer(DataSerializer):
    email:str
    first_name:str=None
    last_name:str=None
    #phone:Phone=Phone()
    address:Address=Address()


@dataclass
class Identification:
    type:str='CPF'
    number:str='00011122233'


@dataclass
class PayerCustomer:
    first_name:str=None
    last_name:str=None
    address:Address=Address()
    phone:Phone=Phone()
    identification:Identification=Identification()

    @classmethod
    def mount_name(cls, name:str):
        """
        Encontra nome e último nome através do nome completo.
        :param name:
        :return:
        """
        customer_name = name.split(' ')
        cls.first_name = name[0]
        cls.last_name = name[-1]

    @classmethod
    def mount_phone(cls, utel:str):
        """
        Preenche dados de telefone.
        :param utel: Telefone do usuário
        :return:
        """
        cls.phone.number = utel[2:]
        cls.phone.area_code = utel[:2]

    @classmethod
    def mount_identification(cls, udoc:str):
        """
        Preenche identificação de acordo com o documento informado.
        :param udoc:
        :return:
        """
        cls.identification.number = udoc
        if len(udoc) == 11:
            cls.identification.type = 'CPF'


@dataclass(init=False)
class PaymentPayer(DataSerializer):
    email:str = None
    last_name:str = None
    first_name:str = None
    entity_type:str = "individual"
    type:str = "customer"
    identification:Identification=Identification()

    @classmethod
    def mount_name(cls, name:str):
        """
        Encontra nome e último nome através do nome completo.
        :param name:
        :return:
        """
        customer_name = name.split(' ')
        cls.first_name = customer_name[0]
        cls.last_name = customer_name[-1]

    @classmethod
    def mount_identification(cls, udoc: str):
        """
        Preenche identificação de acordo com o documento informado.
        :param udoc:
        :return:
        """
        cls.identification.number = udoc
        if len(udoc) == 11:
            cls.identification.type = 'CPF'


@dataclass
class PaymentOrder(DataSerializer):
    #id:int = 0
    type:str = 'mercadopago'


@dataclass
class Payment(DataSerializer):
    payment_method_id:str
    external_reference:str = None  # referencia de relação.
    transaction_amount:int = None  # valor inteiro, em centavos.
    #order:PaymentOrder = PaymentOrder()
    payer:PaymentPayer = PaymentPayer()
    description:str = "Pagamento de produto"
    callback_url:str = None


@dataclass()
class PaymentCard(Payment):
    token:str = None
    issuer_id:int = None
    installments:int = 1


@dataclass()
class PaymentPix(Payment):
    payment_method_id:str = 'pix'


@dataclass
class CardHolder:
    name:str=None
    identification:dict=None


@dataclass
class Card(DataSerializer):
    card_number:str
    security_code:str
    expiration_year:str
    expiration_month:str
    cardholder:CardHolder=CardHolder()
