from datetime import date
from datetime import timedelta

from asc.core import env
from asc.core.ABC.Control import ControlTool
from asc.tools.marketplace.mappings import PaymentGateway
from asc.tools.marketplace.pagarme.api import Pagarme
from asc.tools.marketplace.pagarme.data import Customer
from asc.tools.marketplace.pagarme.data import TransactionPix
from asc.models.marketplace import Payment

class ControlToolHandler(ControlTool):

    def __init__(self):
        super(ControlToolHandler, self).__init__()
        self.__PM = Pagarme()
        self.transaction_items = []

    def recipient_get_list(self) -> list:
        """ Lista recebedores.
        """
        return self.__PM.GET_recipients()

    def recipient_activate(self, id_recipient:str):
        """ Ativa um recebedor.
        """
        return self.__PM.PUT_recipient(
            id_recipient=id_recipient,
            payload={
                'status':'active'
            }
        )

    def recipient_inactivate(self, id_recipient:str, reason:str):
        """ Inativa um recebedor.
        """
        return self.__PM.PUT_recipient(
            id_recipient=id_recipient,
            payload={
                'status':'inactive',
                'status_reason':reason
            }
        )

    def create_transaction_pix(self, customer:Customer, amount:int, **props):
        """ Cria uma transação pix.
        :param customer: Cliente a registrar cobrança.
        :param amount: Valor da cobrança.
        """
        prop_expire = props \
            .get('expires',env.get('marketplace__pix_expires'))
        date_expire = (
            date.today()
            + timedelta(days=int(prop_expire))
        )

        transaction = TransactionPix(
            amount=amount,
            customer=customer,
            pix_expiration_date=date_expire.isoformat()
        )

        payment_post = self.__PM.POST_transaction_pix(transaction)
        print(payment_post)

        payment = Payment()
        payment.id_marketplace_external = payment_post.get('id')
        payment.id_marketplace_client = str(customer.external_id)
        payment.gateway = PaymentGateway.pagarme.name
        payment.transaction_method = 'pix'
        payment.transaction_amount = amount
        payment.meta = {
            'qr':payment_post.get('pix_qr_code'),
            'info':props.get('info')
        }

        self.session.add(payment)
        self.session.flush()
        self.commit()
        return dict(
            id=payment.id_marketplace_payment,
            qr=payment.meta.get('qr'),
            dt_add=payment.dt_add,
            expires=date_expire,
            amount=amount,
            db_payment=payment
        )