from abc import ABC
from typing import List
from dataclasses import dataclass, field
from asc.core import env, log
from asc.core.ABC.Serializer import DataSerializer


@dataclass()
class Customer:
    name:str
    email:str
    country:str = "br"
    type:str = "individual"
    external_id:str = "1"
    documents:List = None
    phone_numbers:List = None


@dataclass()
class Item:
    id:str
    title:str
    quantity:int
    unit_price:str
    tangible:bool=True


@dataclass()
class Address:
    state:str
    city:str
    neighborhood:str
    street:str
    street_number:str
    zipcode:str
    country:str = 'br'


@dataclass()
class Billing:
    name:str
    address:Address


@dataclass()
class Transaction(DataSerializer):
    postback_url:str = env.get('pagarme__url_callback')
    amount:int = 0
    installments:int = 1
    customer:Customer = None
    payment_method:str = None
    
    def __init__(self):
        super(Transaction, self).__init__()

    def add_item(self, item:Item):
        self.items.append(item)


@dataclass()
class TransactionPix(Transaction):
    pix_expiration_date:str = None
    payment_method:str = 'pix'


@dataclass(init=False)
class TransactionCartao(Transaction):
    soft_descriptor:str=env.get('marketplace__soft_descriptor')
    card_number:str
    card_cvv:str
    card_expiration_date:str
    card_holder_name:str
    billing:Billing = None
    items:List[Item] = None
    payment_method = 'credit_card'


@dataclass(init=False)
class TransactionBoleto(Transaction):
    payment_method = 'boleto'

