
import math
import tools
from classes.Valor import Valor as vlrC
from classes.model.OrderTax import OrderTax, OrderTaxData
from classes.DebugSys import DebugSys

debug = DebugSys()

class MarketPlaceTax:

    def __init__(self):
        self.settings = tools.getSettings()
        self.orderTaxData = None
        self.modelOrderTax = OrderTax()
        self.vlrC = vlrC()

    def get_tax_percentage_value(self,vl_percentage: float, vl_order_total: int):
        vl_percentage_result = self.vlrC.getPercentual(
            valor=vl_percentage,
            total=vl_order_total / 100,
            removeDot=True,
            retornaNumero=True
        )

        debug.write('get_tax_percentage_value()',vl_percentage,'% /',vl_order_total,' = ',vl_percentage_result)

        return vl_percentage_result
    
    def add_tax_order_data(self,from_line:str,vl_order_total:float,id_index:int=None):
        """ Incrementa lista de pedidos
        :param id_index: Se informado, o valor é utilizado pelo index 
        informado, ao invez da soma
        """
        #orderTaxData = vars(self.orderTaxData)
        orderTaxData = self.orderTaxData

        # dicionario de valores flutuantes
        tax_entry = orderTaxData.get(f'vl_{from_line}') or [0,0,0]
        debug.write('id_index',id_index,id_index == 0)
        debug.write('tax_entry',tax_entry)
        if id_index or id_index == 0:
            debug.write("id_index existe - tax_entry[id_index]")
            tax_value = tax_entry[id_index]
        else:
            debug.write("id_index ausente - SUM(tax_entry)")
            tax_value = sum(tax_entry)
        debug.write('VL_',from_line,tax_value,type(tax_value))
        
        if orderTaxData.get(f'tp_{from_line}')==1:
            # quando percentual, obtemos o valor aplicado sobre o percentual
            tax_value = self.get_tax_percentage_value(float(tax_value),vl_order_total)
            debug.write('tax_value corrigido - percentual',tax_value,type(tax_value))
        
        tax_value = int(tax_value)\
            if isinstance(tax_value,str)\
            else float(round(tax_value,2))

        self.orderTaxList.append(tax_value) 
        return tax_value

    def get_tax_order_list(self, 
            from_name:str,
            type_name:str,
            vl_order_total: int,
            nr_installments: int
        ):
        """
            Obtém lista de taxas aplicadas em um pedido
        :param from_name: 'pagarme' ou 'marketplace'
        :param type_name: 'pix' ou 'card'
        :param vl_order_total: valor total do pagamento
        :param nr_installments: Número de parcelas
        :return list: lista com taxas em real
        """

        if type_name == 'card':
            
            if nr_installments <= 1: id_installments_index =0
            elif nr_installments <= 6: id_installments_index =1
            else: id_installments_index =2

            debug.write('TAX - CARD from_name',from_name)
            debug.write('TAX - CARD type_name',type_name)
            debug.write('TAX - CARD nr_installments',nr_installments)
            debug.write('TAX - CARD id_installments_index',id_installments_index)
            debug.write('TAX - CARD vl_order_total',vl_order_total)

            # valor da taxa aplicada por processamento
            self.add_tax_order_data(
                from_line=f'{from_name}_{type_name}_process',
                vl_order_total=vl_order_total
            )
            # valor da taxa aplicada por parcela
            self.add_tax_order_data(
                from_line=f'{from_name}_{type_name}_installments',
                vl_order_total=vl_order_total,
                id_index=id_installments_index
            )

        if type_name == 'pix':
            # valor da taxa aplicada por processamento
            self.add_tax_order_data(
                from_line=f'{from_name}_{type_name}',
                vl_order_total=vl_order_total,
                id_index=0
            )
        
        return self.orderTaxList


    def get_tax(self,tp_order_payment:str , vl_order_total: int, only_from: str=None,nr_installments: int=1):
        """
            Obtém valores de taxas como BRL(float), para
            serem registrados em private.order
        :param tp_order_payment: 'pix' ou 'card' - Método de pagamento escolhido
        :param vl_order_total: Valor do pagamento registrado
        :param nr_installments: Número de parcelas
        :return tuple: vl_order_tax_pagarme, vl_order_tax_marketplace
        """
        self.modelOrderTax.sets.order = [['id_order_tax'],'DESC']
        self.orderTaxData: OrderTaxData = self.modelOrderTax.get(
            where={'id_order_tax':self.settings['id_order_tax']})

        # Reseta lista de taxas para nova contagem
        self.orderTaxList = []
        vl_order_tax_pagarme = 0
        vl_order_tax_marketplace = 0

        if not only_from or only_from == 'pagarme':
            vl_order_tax_pagarme = (self.get_tax_order_list(
                from_name='pagarme',
                type_name=tp_order_payment,
                vl_order_total=vl_order_total,
                nr_installments=nr_installments
            ))
            debug.write('vl_order_tax_pagarme',vl_order_tax_pagarme)

        self.orderTaxList = []

        if not only_from or only_from == 'marketplace':
            vl_order_tax_marketplace = (self.get_tax_order_list(
                from_name='marketplace',
                type_name=tp_order_payment,
                vl_order_total=vl_order_total,
                nr_installments=nr_installments
            ))
            debug.write('vl_order_tax_marketplace',vl_order_tax_marketplace)

        if tp_order_payment == 'card':
            return (
                math.floor(sum(vl_order_tax_pagarme)),
                math.floor(sum(vl_order_tax_marketplace))
            )
        else:
            return (
                int(sum(vl_order_tax_pagarme) * 100),
                int(sum(vl_order_tax_marketplace) * 100)
            ) 