import secrets
from hashlib import md5, shake_256
from datetime import datetime, timedelta

from asc.core import env

def get_link_path(id_link:int, sum_date:bool=True) -> str:
    """
    Retorna um caminho seguro para ser compartilhado.
    :param id_link: identificador numérico.
    :param sum_date: soma data ao código identificador.
    :return: link sem barra.
    """

    salt_str = env.get('storage__salt_str', 'abcd')
    salt_int = int(env.get('storage__salt_int', 7))
    salt_day = int(env.get('storage__salt_day', 1))

    link_date = int(str(round(
        (datetime.now() + timedelta(days=salt_day)).timestamp())
    )[-10:].zfill(10))

    link_code = id_link + salt_int

    if sum_date:
        link_code += link_date

    link_salt = f'{link_code*salt_int:016}'[-16:]
    link_hash = md5(str(f'{link_code}{salt_str}')
                    .encode('utf-8')).hexdigest()

    link_path = 'asc_public_' \
                f'{link_hash}' \
                f'{secrets.token_hex(16)}' \
                f'{link_salt}' \
                f'{shake_256(link_hash.encode()).hexdigest(64)}'

    return link_path